package ruizide

import (
	"encoding/json"
	"eta/eta_data_analysis/models"
	"eta/eta_data_analysis/utils"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/xuri/excelize/v2"
	"os"
	"path/filepath"
	"regexp"
	"strings"
)

var (
	OilDemandSignalsWeeklyReportConfigMap = map[string]config{
		"Road Index": {
			Frequency:       "日度",
			Unit:            "%",
			IndexNameColOne: "Index",
			IndexNameColTwo: "Index 7DMA",
			IndexPrefix:     "Road Index",
			BeginRow:        4,
			Rows:            5,
			IndexRow:        2,
			DataOneRow:      3,
			DataTwoRow:      4,
			DateRow:         1,
		},
		"Road Active Fleet": {
			Frequency:       "日度",
			Unit:            "辆",
			IndexNameColOne: "Active Fleet",
			IndexNameColTwo: "Active Fleet 7DMA",
			IndexPrefix:     "Road Active Fleet",
			BeginRow:        4,
			Rows:            5,
			IndexRow:        2,
			DataOneRow:      3,
			DataTwoRow:      4,
			DateRow:         1,
		},
		"Aviation Index": {
			Frequency:       "日度",
			Unit:            "%",
			IndexNameColOne: "Index",
			IndexNameColTwo: "Index 7DMA",
			IndexPrefix:     "Aviation Index",
			BeginRow:        4,
			Rows:            5,
			IndexRow:        2,
			DataOneRow:      3,
			DataTwoRow:      4,
			DateRow:         1,
		},
		"Aviation Active Fleet": {
			Frequency:       "日度",
			Unit:            "辆",
			IndexNameColOne: "Active Fleet",
			IndexNameColTwo: "Active Fleet 7DMA",
			IndexPrefix:     "Aviation Active Fleet",
			BeginRow:        4,
			Rows:            5,
			IndexRow:        2,
			DataOneRow:      3,
			DataTwoRow:      4,
			DateRow:         1,
		},
		"Demand - Gasoline": {
			Frequency:       "日度",
			Unit:            "千桶每天",
			IndexNameColOne: "Demand",
			IndexNameColTwo: "Demand 7DMA",
			IndexPrefix:     "Gasoline Demand",
			BeginRow:        4,
			Rows:            6,
			IndexRow:        3,
			DataOneRow:      4,
			DataTwoRow:      5,
			DateRow:         1,
		},
		"Demand - Diesel": {
			Frequency:       "日度",
			Unit:            "千桶每天",
			IndexNameColOne: "Demand",
			IndexNameColTwo: "Demand 7DMA",
			IndexPrefix:     "Demand Diesel",
			BeginRow:        4,
			Rows:            6,
			IndexRow:        3,
			DataOneRow:      4,
			DataTwoRow:      5,
			DateRow:         1,
		},
		"Demand - Jet Fuel": {
			Frequency:       "日度",
			Unit:            "千桶每天",
			IndexNameColOne: "Demand",
			IndexNameColTwo: "Demand 7DMA",
			IndexPrefix:     "Demand Jet Fuel",
			BeginRow:        4,
			Rows:            6,
			IndexRow:        3,
			DataOneRow:      4,
			DataTwoRow:      5,
			DateRow:         1,
		},
		"Demand - Maritime Bunker": {
			Frequency:       "日度",
			Unit:            "千桶每天",
			IndexNameColOne: "Demand",
			IndexNameColTwo: "Demand 7DMA",
			IndexPrefix:     "Demand Maritime Bunker",
			BeginRow:        4,
			Rows:            6,
			IndexRow:        3,
			DataOneRow:      4,
			DataTwoRow:      5,
			DateRow:         1,
		},
	}
)

type config struct {
	Frequency       string
	Unit            string
	IndexNameColOne string
	IndexNameColTwo string
	IndexPrefix     string
	BeginRow        int
	Rows            int
	IndexRow        int
	DataOneRow      int
	DataTwoRow      int
	DateRow         int
}
type OilDemandSignalsWeeklyReportProcessor struct {
}

func (p *OilDemandSignalsWeeklyReportProcessor) Process(tableName string) (err error) {
	var fileName string
	// 解析表格
	fileName = tableName + "_" + utils.GetCurrentYearMonth() + ".xlsx"
	filePath := filepath.Join(excelDir, fileName)
	if _, err = os.Stat(filePath); os.IsNotExist(err) {
		utils.FileLog.Error("文件不存在: %v", err)
		return
	}
	// 打开 Excel 文件
	file, err := excelize.OpenFile(filePath)
	if err != nil {
		utils.FileLog.Error("无法打开 Excel 文件: %v", err)
		return
	}
	sheetNames := file.GetSheetList()
	classifyIds, err := dealClassify(tableName, sheetNames)
	// 获取所有工作表
	if err != nil {
		utils.FileLog.Error("获取分类Id失败: %v", err)
		return
	}
	//按照sheet页来处理数据,一个sheet发送一次服务器
	for _, sheetName := range sheetNames {
		var indexMap = make(map[string]*models.IndexInfo)
		if sheetName == "Content" {
			utils.FileLog.Info("跳过工作表:", sheetName)
			continue
		}
		utils.FileLog.Info("读取工作表: %s\n", sheetName)
		// 获取工作表的最大行数
		dataRows, excelErr := file.GetRows(sheetName) // 直接获取所有行数据
		if excelErr != nil {
			utils.FileLog.Error("获取工作表数据时出错: %v", excelErr)
			continue
		}
		sheetConfig := OilDemandSignalsWeeklyReportConfigMap[sheetName]
		for _, rowData := range dataRows[sheetConfig.BeginRow:] {
			//处理数据不全的情况
			less := sheetConfig.Rows - len(rowData)
			if less > 0 {
				for i := 0; i < less; i++ {
					rowData = append(rowData, "")
				}
			}
			// 匹配非数字字符
			//re := regexp.MustCompile(`^[-+]?\d*\.?\d+$`)

			classifyId := classifyIds[sheetName]
			// step_2: 指标
			// 指标名称
			indexNameOne := sheetName + "/" + rowData[sheetConfig.IndexRow] + "/" + sheetConfig.IndexNameColOne
			indexNameTwo := sheetName + "/" + rowData[sheetConfig.IndexRow] + "/" + sheetConfig.IndexNameColTwo
			// 生成指标编码
			indexCodeOne := getIndexId(sheetName, strings.ReplaceAll(strings.ToLower(rowData[len(rowData)-3]), " ", ""), sheetConfig.IndexNameColOne)
			indexCodeTwo := getIndexId(sheetName, strings.ReplaceAll(strings.ToLower(rowData[len(rowData)-3]), " ", ""), sheetConfig.IndexNameColTwo)
			dataTime := rowData[sheetConfig.DateRow]
			date, _, convertErr := utils.ConvertDateFormat(dataTime)
			if convertErr != nil {
				utils.FileLog.Error(fmt.Sprintf("转换时间数据失败,index_code:%s,time_value:%s err:%v", indexCodeOne, dataTime, convertErr))
				continue
			}
			if rowData[sheetConfig.DataOneRow] != "" {
				valueOne, parseErr := decimal.NewFromString(strings.ReplaceAll(rowData[sheetConfig.DataOneRow], ",", ""))
				if parseErr != nil {
					utils.FileLog.Error(fmt.Sprintf("转换data数据失败,index_code:%s,data_value:%s err:%v", indexCodeOne, rowData[sheetConfig.DataOneRow], err))
					continue
				} else {
					if index, ok := indexMap[indexCodeOne]; ok {
						if index.StartDate.After(date) {
							index.StartDate = date
						}
						if index.EndDate.Before(date) {
							index.EndDate = date
							index.LatestValue = valueOne
						}
						index.DataList = append(index.DataList, models.IndexData{
							DataTime: date,
							Value:    valueOne,
						})
					} else {
						indexMap[indexCodeOne] = &models.IndexInfo{
							IndexName:   indexNameOne,
							IndexCode:   indexCodeOne,
							Frequency:   sheetConfig.Frequency,
							Unit:        sheetConfig.Unit,
							StartDate:   date,
							EndDate:     date,
							LatestValue: valueOne,
							ClassifyId:  classifyId,
							DataList: []models.IndexData{
								{
									DataTime: date,
									Value:    valueOne,
								},
							},
						}
					}
				}
			}
			if rowData[sheetConfig.DataTwoRow] != "" {
				valueTwo, parseErr := decimal.NewFromString(strings.ReplaceAll(rowData[sheetConfig.DataTwoRow], ",", ""))
				if parseErr != nil {
					utils.FileLog.Error(fmt.Sprintf("转换data数据失败,index_code:%s,data_value:%s err:%v", indexCodeTwo, rowData[sheetConfig.DataTwoRow], err))
					continue
				} else {
					if index, ok := indexMap[indexCodeTwo]; ok {
						if index.StartDate.After(date) {
							index.StartDate = date
						}
						if index.EndDate.Before(date) {
							index.EndDate = date
							index.LatestValue = valueTwo
						}
						index.DataList = append(index.DataList, models.IndexData{
							DataTime: date,
							Value:    valueTwo,
						})
					} else {
						indexMap[indexCodeTwo] = &models.IndexInfo{
							IndexName:   indexNameTwo,
							IndexCode:   indexCodeTwo,
							Frequency:   sheetConfig.Frequency,
							Unit:        sheetConfig.Unit,
							StartDate:   date,
							EndDate:     date,
							LatestValue: valueTwo,
							ClassifyId:  classifyId,
							DataList: []models.IndexData{
								{
									DataTime: date,
									Value:    valueTwo,
								},
							},
						}
					}
				}
			}
		}
		utils.FileLog.Info("读取工作表完成:", sheetName)
		// 新增数据源指标数据
		if len(indexMap) > 0 {
			for _, index := range indexMap {
				pars := chunkIndexData(index, rzdBatchSize)
				for _, par := range pars {
					// 转换成json
					marshal, parseErr := json.Marshal(par)
					if parseErr != nil {
						utils.FileLog.Error("json.Marshal err: %v", parseErr)
						return parseErr
					}
					// 发送 HTTP POST 请求
					_, err = utils.HttpPostRequest(utils.EDB_LIB_URL+utils.ADD_BATCH_RZD_DATA, string(marshal), "application/json")
					if err != nil {
						utils.FileLog.Error("postEdbLib err: %v", err)
						return err
					}
				}
			}
		}
	}
	return
}
func isValidNumber(s string) bool {
	// 匹配整数或浮点数（支持正负号）
	re := regexp.MustCompile(`^[-+]?\d*\.?\d+$`)
	return re.MatchString(s)
}
func chunkIndexData(index *models.IndexInfo, rzdBatchSize int) (pars []models.IndexInfo) {
	var chunks int
	total := len(index.DataList)
	count := total % rzdBatchSize
	if count > 0 {
		chunks = (total + rzdBatchSize - count) / rzdBatchSize
	} else {
		chunks = total / rzdBatchSize
	}
	for i := 0; i < chunks; i++ {
		start := i * rzdBatchSize
		end := start + rzdBatchSize
		if end > total {
			end = total
		}
		pars = append(pars, models.IndexInfo{
			IndexName:   index.IndexName,
			IndexCode:   index.IndexCode,
			LatestValue: index.LatestValue,
			StartDate:   index.StartDate,
			EndDate:     index.EndDate,
			Frequency:   index.Frequency,
			Unit:        index.Unit,
			ClassifyId:  index.ClassifyId,
			DataList:    index.DataList[start:end],
		})
	}
	return
}
