package ruizide

import (
	"encoding/json"
	"eta/eta_data_analysis/models"
	"eta/eta_data_analysis/utils"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/xuri/excelize/v2"
	"os"
	"path/filepath"
	"strings"
)

const (
	MCSheetName    = "ExportCubeBrowser 1"
	OilMarketCube  = "Oil Market Cube"
	UpstreamSupply = "Upstream Supply"
	Balances       = "Balances"
	ProductsDemand = "Products Demand"
	Geography      = "Geography"
	MCBeginRow     = 2
	MCRows         = 3
)

type MCPConfig struct {
	Frequency          string
	Unit               string
	IndexNameColSuffix string
	IndexNameColPrefix string
	YearRow            int
	DataRow            int
	MonthRow           int
	SuffixName         string
	CodeArea           string
	PrefixStr          string
	ClassifyName       string
}

var (
	OilMarketCubeConfigMap = map[string]MCPConfig{
		"Oil_Market_Cube_Upstream_Supply_Oil_Quality_API": {
			Frequency:          "月度",
			Unit:               "°",
			IndexNameColSuffix: "Oil Quality",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			SuffixName:         "API",
			ClassifyName:       UpstreamSupply,
			CodeArea:           "API",
		},
		"Oil_Market_Cube_Upstream_Supply_Oil_Quality_Sulphur": {
			Frequency:          "月度",
			Unit:               "%wt",
			IndexNameColSuffix: "Oil Quality",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			SuffixName:         "Sulphur",
			ClassifyName:       UpstreamSupply,
			CodeArea:           "Sulphur",
		},
		"Oil_Market_Cube_Upstream_Supply_Capacity_Capacity": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Capacity",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_Production": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Production",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_Production_Wo_Seasonality": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Production Wo Seasonality",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_OPEC_Policy_Reference_Production": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Reference Production",
			IndexNameColPrefix: "OPEC_Policy",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_OPEC_Policy_Target_Production": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Target Production",
			IndexNameColPrefix: "OPEC_Policy",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_OPEC_Policy_Target_Cut": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Target Cut",
			IndexNameColPrefix: "OPEC_Policy",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_OPEC_Policy_Actual_Cut": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Actual Cut",
			IndexNameColPrefix: "OPEC_Policy",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_OPEC_Policy_Compliance": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Compliance",
			IndexNameColPrefix: "OPEC_Policy",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Upstream_Supply_OPEC_Policy_Production_Subject_To_Cut": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Production Subject To Cut",
			IndexNameColPrefix: "OPEC_Policy",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       UpstreamSupply,
		},
		"Oil_Market_Cube_Products_Demand_Products_Demand_Mean": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Products Demand Mean (1.0 DG) ",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       ProductsDemand,
			CodeArea:           "1.0dg",
			PrefixStr:          "Products Demand Mean",
		},
		"Oil_Market_Cube_Products_Demand_Products_Demand+Sigma": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Products Demand +Sigma (2.2 DG) ",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       ProductsDemand,
			CodeArea:           "2.2dg",
			PrefixStr:          "Products Demand Sigma",
		},
		"Oil_Market_Cube_Products_Demand_Products_Demand-Sigma": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Products Demand -Sigma (1.8 DG) ",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       ProductsDemand,
			CodeArea:           "1.8dg",
			PrefixStr:          "Products Demand Sigma",
		},
		"Oil_Market_Cube_Balances_Total_Liquids_Balances": {
			Frequency:          "月度",
			Unit:               "kbbl/d",
			IndexNameColSuffix: "Total Liquids Balances",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       Balances,
		},
		"Oil_Market_Cube_Geography_Latitude": {
			Frequency:          "月度",
			Unit:               "",
			IndexNameColSuffix: "Latitude",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       Geography,
		},
		"Oil_Market_Cube_Geography_Longitude": {
			Frequency:          "月度",
			Unit:               "",
			IndexNameColSuffix: "Longitude",
			MonthRow:           1,
			DataRow:            2,
			YearRow:            0,
			ClassifyName:       Geography,
		},
	}
)

type OilMarketCubeProcessor struct {
}

func (p *OilMarketCubeProcessor) Process(tableName string) (err error) {
	var fileName string
	// 解析表格
	fileName = tableName + "_" + utils.GetCurrentYearMonth() + ".xlsx"
	filePath := filepath.Join(excelDir, fileName)
	if _, err = os.Stat(filePath); os.IsNotExist(err) {
		utils.FileLog.Error("文件不存在: %v", err)
		return
	}
	// 打开 Excel 文件
	file, err := excelize.OpenFile(filePath)
	if err != nil {
		utils.FileLog.Error("无法打开 Excel 文件: %v", err)
		return
	}

	//按照sheet页来处理数据,一个sheet发送一次服务器
	var indexMap = make(map[string]*models.IndexInfo)
	// 获取工作表的最大行数
	dataRows, excelErr := file.GetRows(MCSheetName) // 直接获取所有行数据
	if excelErr != nil {
		utils.FileLog.Error("获取工作表数据时出错: %v", excelErr)
		return
	}
	if sheetConfig, ok := OilMarketCubeConfigMap[tableName]; ok {
		classifyIds, classifyErr := dealClassify(OilMarketCube, []string{sheetConfig.ClassifyName})
		// 获取所有工作表
		if classifyErr != nil {
			utils.FileLog.Error("获取分类Id失败: %v", classifyErr)
			return
		}
		// 匹配非数字字符
		classifyId := classifyIds[sheetConfig.ClassifyName]
		indexMap, err = MarketCubeDataHandler(dataRows, sheetConfig, classifyId)
		// 新增数据源指标数据
		if len(indexMap) > 0 {
			for _, index := range indexMap {
				pars := chunkIndexData(index, rzdBatchSize)
				for _, par := range pars {
					// 转换成json
					marshal, parseErr := json.Marshal(par)
					if parseErr != nil {
						utils.FileLog.Error("json.Marshal err: %v", parseErr)
						return parseErr
					}
					// 发送 HTTP POST 请求
					_, err = utils.HttpPostRequest(utils.EDB_LIB_URL+utils.ADD_BATCH_RZD_DATA, string(marshal), "application/json")
					if err != nil {
						utils.FileLog.Error("postEdbLib err: %v", err)
						return err
					}
				}
			}
		}
	}
	return
}
func MarketCubeDataHandler(dataRows [][]string, sheetConfig MCPConfig, classifyId int) (indexMap map[string]*models.IndexInfo, err error) {
	indexMap = make(map[string]*models.IndexInfo)
	var yearStr string
	for _, rowData := range dataRows[MCBeginRow : len(dataRows)-1] {
		less := MCRows - len(rowData)
		if less > 0 {
			for i := 0; i < less; i++ {
				rowData = append(rowData, "")
			}
		}
		if rowData[sheetConfig.YearRow] != "" {
			yearStr = rowData[sheetConfig.YearRow]
		}
		// step_2: 指标
		// 指标名称
		var indexName, indexCode string
		if sheetConfig.IndexNameColPrefix == "" {
			indexName = sheetConfig.ClassifyName + "/" + sheetConfig.IndexNameColSuffix
			if sheetConfig.SuffixName != "" {
				indexName = indexName + "/" + sheetConfig.SuffixName
			}
		} else {
			indexName = sheetConfig.ClassifyName + "/" + sheetConfig.IndexNameColPrefix + "/" + sheetConfig.IndexNameColSuffix
		}
		// 生成指标编码
		var area, prefix string
		if sheetConfig.PrefixStr == "" {
			prefix = sheetConfig.IndexNameColPrefix
		} else {
			prefix = sheetConfig.PrefixStr
		}
		if sheetConfig.CodeArea != "" {
			area = sheetConfig.CodeArea
		} else {
			area = sheetConfig.IndexNameColSuffix
		}
		if sheetConfig.IndexNameColPrefix != "" {
			indexCode = getIndexId(fmt.Sprintf("%s %s", sheetConfig.ClassifyName, prefix), strings.ReplaceAll(strings.ToLower(area), " ", ""), "")
		} else {
			indexCode = getIndexId(sheetConfig.ClassifyName, strings.ReplaceAll(strings.ToLower(area), " ", ""), "")
		}
		dataTime := fmt.Sprintf("%s-%s", yearStr, rowData[sheetConfig.MonthRow])
		if dataTime == "0" {
			continue
		}
		date, _, convertErr := utils.ConvertDateFormatForMC(dataTime)
		if convertErr != nil {
			utils.FileLog.Error(fmt.Sprintf("转换时间数据失败,index_code:%s,time_value:%s err:%v", indexCode, dataTime, convertErr))
			continue
		}
		if rowData[sheetConfig.DataRow] != "" {
			value, parseErr := decimal.NewFromString(rowData[sheetConfig.DataRow])
			if parseErr != nil {
				utils.FileLog.Error(fmt.Sprintf("转换data数据失败,index_code:%s,data_value:%s err:%v", indexCode, rowData[sheetConfig.DataRow], err))
				continue
			} else {
				if index, ok := indexMap[indexCode]; ok {
					if index.StartDate.After(date) {
						index.StartDate = date
					}
					if index.EndDate.Before(date) {
						index.EndDate = date
						index.LatestValue = value
					}
					index.DataList = append(index.DataList, models.IndexData{
						DataTime: date,
						Value:    value,
					})
				} else {
					indexMap[indexCode] = &models.IndexInfo{
						IndexName:   indexName,
						IndexCode:   indexCode,
						Frequency:   sheetConfig.Frequency,
						Unit:        sheetConfig.Unit,
						StartDate:   date,
						EndDate:     date,
						LatestValue: value,
						ClassifyId:  classifyId,
						DataList: []models.IndexData{
							{
								DataTime: date,
								Value:    value,
							},
						},
					}
				}
			}
		}
	}
	return
}
